Option Explicit 

'Specify Zip Code
'SLO - 93401
'Houston - 77002
'Lafayette - 70501
'Orlando - 32801

Dim strZipCode : strZipCode = "77002"

Dim strRespXml, _
	strCondition, _
	strTempF, _
	strObservationTime, _
	strLocation, _
	strFeelsLike, _
	strWindMph

Dim bRet
bRet = RetrieveWeatherDataForZipCode(strZipCode, _
									 strRespXml, _
									 strCondition, _
									 strTempF, _
									 strObservationTime, _
									 strLocation, _
									 strFeelsLike, _
									 strWindMph)
									 
'output results if successful
If bRet = True Then
	WScript.Echo "Response XML: " & vbCrLf & strRespXml & vbCrLf
	WScript.Echo "Timestamp: " & strObservationTime
	WScript.Echo "Condition: " & strCondition
	WScript.Echo "Temp: " & strTempF
	WScript.Echo "Feels Like: "& strFeelsLike
	WScript.Echo "Wind Speed: " & strWindMph
	WScript.Echo "Location: " & strLocation
Else
	WScript.Echo "Load Unsuccessful"
End If




'**********************************************************************************
' RetrieveWeatherDataForZipCode
' Notes: Retrieve current weather data items from the Weather Underground API
'		 Return True on success, False if an error occured
'**********************************************************************************
Function RetrieveWeatherDataForZipCode( ByVal strZipCodeIn, _
										ByRef strResponseXml, _
										ByRef strCondition, _
										ByRef strTempF, _
										ByRef strObservationTime, _
										ByRef strLocation, _
										ByRef strFeelsLike, _
										ByRef strWindMph)

	'default return value to False
	RetrieveWeatherDataForZipCode = False
	
	'Create XML Dom object
	Dim xmlDoc : Set xmlDoc = CreateObject("Msxml2.DOMDocument")
	xmlDoc.async = False
	
	'Read Weather Underground API Key
	Dim strWunderApiKey
	strWunderApiKey = GetWeatherUndergroundAPIKey()
	
	'Specify Weather Underground query - This is the current conditions query
	Dim strUrl
	strUrl = "http://api.wunderground.com/api/" & strWunderApiKey & "/conditions/q/" & strZipCodeIn & ".xml"
	
	'Load XML document from URL
	If xmlDoc.load(strUrl) Then

		'Load successful - Begin parsing response
		'Get specific attribute values out of XML using XPath queries
		strResponseXml = xmlDoc.xml
		strObservationTime = xmlDoc.SelectSingleNode("/response/current_observation/observation_time_rfc822").text
	    strCondition = xmlDoc.SelectSingleNode("/response/current_observation/weather").text
	    strTempF = xmlDoc.SelectSingleNode("/response/current_observation/temp_f").text
	    strFeelsLike = xmlDoc.SelectSingleNode("/response/current_observation/feelslike_f").text
		strLocation = xmlDoc.selectSingleNode("/response/current_observation/display_location/full").text
		strWindMph = xmlDoc.selectSingleNode("/response/current_observation/wind_mph").text

		'success so set return value to True
		RetrieveWeatherDataForZipCode = True
	End If
	
End Function


'**********************************************************************************
' GetWeatherUndergroundAPIKey
' Notes: Reads Weather Underground API key from known file location
'**********************************************************************************
Function GetWeatherUndergroundAPIKey()
	Dim strKeyPath : strKeyPath = "C:\Weather Underground API\key.txt"
	
	Dim fso : Set fso = CreateObject("Scripting.FileSystemObject")
	
	If fso.FileExists(strKeyPath) Then
		Dim objFile
		Set objFile = fso.OpenTextFile(strKeyPath)
		
		GetWeatherUndergroundAPIKey = objFile.ReadAll
	End If
	
End Function
