﻿Public Class WeatherRetrieval
    '---------------------------------------------------------------------------------------------------
    'Specify Weather Underground API Key
    Dim strWunderApiKey = GetWeatherUndergroundAPIKey()

    'Specify weather facility type
    Const strFacType = "WEATHER"

    'Update rate
    Const iUpdateRate = 120

    'Weather UDCs
    Const strConditionUdc = "WCOND"         'Current Conditions - String
    Const strTempUdc = "WTEMP"              'Current Temp - Deg F
    Const strFeelsLikeUdc = "WFEELSLK"      'Feels Like Temp - Deg F
    Const strWindSpeedUdc = "WWINDSP"       'Wind Speed - mph
    Const strLocationUdc = "WLOC"           'Location - String

    'Weather query override - set to false to disable the actual weather query - use for troubleshooting
    Const bQueryWeather = True
    '---------------------------------------------------------------------------------------------------

    Dim globFunc As New CXSCRIPTLib.GlobalFunctions
    Dim facilities As New CXSCRIPTLib.Facilities

    '**********************************************************************************
    ' GetWeatherUndergroundAPIKey
    ' Notes: Reads Weather Underground API key from known file location
    '**********************************************************************************
    Function GetWeatherUndergroundAPIKey()
        Dim strKeyPath : strKeyPath = "C:\Weather Underground API\key.txt"

        Dim fso : fso = CreateObject("Scripting.FileSystemObject")

        If fso.FileExists(strKeyPath) Then
            Dim objFile
            objFile = fso.OpenTextFile(strKeyPath)

            GetWeatherUndergroundAPIKey = objFile.ReadAll
        Else
            GetWeatherUndergroundAPIKey = ""
        End If
    End Function

    '**********************************************************************************
    ' CreateFacilityandPoints
    ' Notes: Create zip code facility and associated points
    '**********************************************************************************
    Function CreateFacilityandPoints(strZipCode, strUisSvc)
        Dim strResult : strResult = ""

        strResult = CreateFacility(strZipCode, strUisSvc)

        'If creating the facility succeeds then create points
        If strResult = "" Then
            strResult = CreatePoints(strUisSvc, strZipCode)
        End If

        IsPntCvsQueueEmpty(strUisSvc)

        CreateFacilityandPoints = strResult
    End Function

    '**********************************************************************************
    ' IsFacilityCreated
    ' Notes: Returns true if the provided facility ID exists
    '**********************************************************************************
    Function IsFacilityCreated(strCvsSvc, strFacId)
        On Error Resume Next
        err.Clear()

        'Query facility service for facility id
        Dim arrTagList As New Object
        facilities.GetFacilityTagList(strCvsSvc, "facility_id=" & strFacId, arrTagList)

        'Check to see if anything was returned or if there was an error
        If ubound(arrTagList) < 0 Or err.Number <> 0 Then
            IsFacilityCreated = False
        Else
            IsFacilityCreated = True
        End If

        On Error GoTo 0
    End Function

    '**********************************************************************************
    ' CreateFacility
    ' Notes: creates weather facilities for new zip codes
    '**********************************************************************************
    Function CreateFacility(strFacId, strUisSvc)
        On Error Resume Next

        'Create facility client
        Dim facClient : facClient = CreateObject("CxFac.FacClient")

        'Check to see if the facClient is connected - if not connect
        If Not facClient.IsConnected Then
            'Determine facility service
            Dim strFacSvc : strFacSvc = globFunc.GetAssociatedFacName(strUisSvc)

            If err.Number <> 0 Then
                CreateFacility = "Unable to obtain facility service from UIS (" & strUisSvc & ")"
                Exit Function
            End If

            err.Clear()

            'Connect to facility service
            facclient.Connect(strFacSvc)

            'If there was an error, log it and exit
            If err.Number <> 0 Then
                CreateFacility = "Unable to connect to service (" & strFacSvc & ")"
                Exit Function
            End If
        End If

        'Get a blank facility record and build up a new facility record to add
        Dim newFacRecord : newFacRecord = facClient.GetEmptyRecordXml()
        newFacRecord = facClient.SetRecordXMLAttribute(newFacRecord, "facility_id", strFacId)
        newFacRecord = facClient.SetRecordXMLAttribute(newFacRecord, "facility_type", strFacType)
        newFacRecord = facClient.SetRecordXMLAttribute(newFacRecord, "facility_siteservice", strUisSvc)

        'Add new facility
        facClient.AddFacilityRecord(newFacRecord)

        'If there was an error, log it and exit	
        If Err.Number <> 0 Then
            CreateFacility = "Error creating facility. " & vbCrLf & Err.Description
            Exit Function
        End If

        'Log success
        CreateFacility = ""

        On Error GoTo 0
    End Function

    '**********************************************************************************
    ' CreatePoints
    ' Notes: Create weather points for facility
    '**********************************************************************************
    Function CreatePoints(strUisSvc, strFacId)
        Dim strResult : strResult = ""

        If strResult = "" Then
            strResult = CreatePoint(strUisSvc, strFacId, strConditionUdc, "Condition", "SI", "")
        End If

        If strResult = "" Then
            strResult = CreatePoint(strUisSvc, strFacId, strTempUdc, "Temperature", "AI", "F")
        End If

        If strResult = "" Then
            strResult = CreatePoint(strUisSvc, strFacId, strFeelsLikeUdc, "Feels Like", "AI", "F")
        End If

        If strResult = "" Then
            strResult = CreatePoint(strUisSvc, strFacId, strWindSpeedUdc, "Wind Speed", "AI", "mi/hr")
        End If

        If strResult = "" Then
            strResult = CreatePoint(strUisSvc, strFacId, strLocationUdc, "Location", "SI", "")
        End If

        CreatePoints = strResult
    End Function

    '**********************************************************************************
    ' IsPointCreated
    ' Notes: Returns true if the provided point exists
    '**********************************************************************************
    Function IsPointCreated(strUisSvc, strFacId, strUdc)
        On Error Resume Next
        err.Clear()

        Dim arrTagList As New Object
        globFunc.GetTagList(strUisSvc, "facility=" & strFacId & ";udc=" & strUdc, True, arrTagList)

        If ubound(arrTagList) < 0 Or err.Number <> 0 Then
            IsPointCreated = False
        Else
            IsPointCreated = True
        End If

        On Error GoTo 0
    End Function

    '**********************************************************************************
    ' CreatePoint
    ' Notes: Creates point
    '**********************************************************************************
    Function CreatePoint(strUisSvc, strFacId, strUdc, strPointDesc, strPointType, strPriUnits)
        On Error Resume Next
        err.Clear()

        'Check to see if point is created
        If Not IsPointCreated(strUisSvc, strFacId, strUdc) Then
            'Get PNT service name from UIS service
            Dim pntClient : pntClient = CreateObject("CxPnt.PntClient")
            Dim strPntSvc : strPntSvc = globFunc.GetAssociatedPointCfgName(strUisSvc)

            If err.Number <> 0 Then
                CreatePoint = "Unable to obtain point service from UIS (" & strUisSvc & ")"
                Exit Function
            End If

            err.Clear()

            'Connect to PNT service
            pntClient.Connect(strPntSvc)

            'If there was an error, log it and exit
            If err.Number <> 0 Then
                CreatePoint = "Unable to connect to service (" & strPntSvc & ")"
                Exit Function
            End If

            err.Clear()

            'Build new point record		
            Dim strNewPointRecXml : strNewPointRecXml = pntClient.GetEmptyPointXml()
            strNewPointRecXml = pntClient.SetPointXMLAttribute(strNewPointRecXml, "siteservice", strUisSvc)
            strNewPointRecXml = pntClient.SetPointXMLAttribute(strNewPointRecXml, "pointidlong", strFacId & "_" & strUdc)
            strNewPointRecXml = pntClient.SetPointXMLAttribute(strNewPointRecXml, "description", strPointDesc)
            strNewPointRecXml = pntClient.SetPointXMLAttribute(strNewPointRecXml, "reportvhs", "1")
            strNewPointRecXml = pntClient.SetPointXMLAttribute(strNewPointRecXml, "facilityid", strFacId)
            strNewPointRecXml = pntClient.SetPointXMLAttribute(strNewPointRecXml, "uniformdatacode", strUdc)
            strNewPointRecXml = pntClient.SetPointXMLAttribute(strNewPointRecXml, "pointdatatype", strPointType)
            strNewPointRecXml = pntClient.SetPointXMLAttribute(strNewPointRecXml, "priunits", strPriUnits)
            strNewPointRecXml = pntClient.SetPointXMLAttribute(strNewPointRecXml, "pointscheme", "0")

            'Add point to PNT
            pntClient.AddPointRecord(strNewPointRecXml)

            If err.Number <> 0 Then
                CreatePoint = "Error creating point " & strFacId & "_" & strUdc & "."
            Else
                CreatePoint = ""
            End If
        End If
    End Function

    '**********************************************************************************
    ' UpdateWeather
    ' Notes: Updates values for all weather points
    '**********************************************************************************
    Sub UpdateWeather(strCvsSvc)
        Dim arrFacTags As New Object
        facilities.GetFacilityTagList(strCvsSvc, "facility_type=WEATHER", arrFacTags)

        Dim strFacTag
        For Each strFacTag In arrFacTags
            Call GetWeatherForFacTag(strFacTag)
        Next
    End Sub

    '**********************************************************************************
    ' GetWeatherForFacTag
    ' Notes: Gets weather data for the specified weather facility
    '**********************************************************************************
    Sub GetWeatherForFacTag(strFacTag)
        On Error Resume Next


        'Check to see if it's time to update weather facility	
        If Not ShouldFacilityUpdate(strFacTag) Then
            Exit Sub
        End If

        'Get zip code from facility tag
        Dim strZipCode : strZipCode = Right(strFacTag, Len(strFacTag) - InStr(strFacTag, "::") - 1)

        'Create XML Dom object
        Dim xmlDoc : xmlDoc = CreateObject("Msxml2.DOMDocument")
        xmlDoc.async = False

        'Specify Wunder query - This is the current conditions query
        Dim strUrl : strUrl = "http://api.wunderground.com/api/" & strWunderApiKey & "/conditions/q/" & strZipCode & ".xml"

        If Not bQueryWeather Then
            Exit Sub
        End If

        err.Clear()
        'Load weather query URL into XML object.		
        If xmlDoc.load(strUrl) Then
            'Load was successful - Begin parsing response
            Dim strCondition, strTempF, strObservationTime, strLocation, strFeelsLike, strWindMph

            strObservationTime = xmlDoc.SelectSingleNode("/response/current_observation/observation_time_rfc822").text
            strCondition = xmlDoc.SelectSingleNode("/response/current_observation/weather").text
            strTempF = xmlDoc.SelectSingleNode("/response/current_observation/temp_f").text
            strFeelsLike = xmlDoc.SelectSingleNode("/response/current_observation/feelslike_f").text
            strLocation = xmlDoc.selectSingleNode("/response/current_observation/display_location/full").text
            strWindMph = xmlDoc.selectSingleNode("/response/current_observation/wind_mph").text

            If err.Number = 0 Then
                globFunc.EnableLiveMode(True)

                'Issue setpoints
                globFunc.SetPointEx(Replace(strFacTag, "::", ":") & "_" & strConditionUdc, strCondition, Now(), 0, 0)
                globFunc.SetPointEx(Replace(strFacTag, "::", ":") & "_" & strTempUdc, strTempF, Now(), 0, 0)
                globFunc.SetPointEx(Replace(strFacTag, "::", ":") & "_" & strFeelsLikeUdc, strFeelsLike, Now(), 0, 0)
                globFunc.SetPointEx(Replace(strFacTag, "::", ":") & "_" & strLocationUdc, strLocation, Now(), 0, 0)
                globFunc.SetPointEx(Replace(strFacTag, "::", ":") & "_" & strWindSpeedUdc, strWindMph, Now(), 0, 0)
            End If
        End If
    End Sub

    '**********************************************************************************
    ' ShouldFacilityUpdate
    ' Notes: Check to see if it's time to update the weather facility
    '**********************************************************************************
    Function ShouldFacilityUpdate(strFacTag)
        ShouldFacilityUpdate = True

        'Create Points object
        Dim objPoints : objPoints = CreateObject("CxScript.Points")
        objpoints.EnableUpdate(True)

        '-----------------------------------------------------------------------------------------------------------------------------------
        'The following code is used to throttle weather queries.  
        'The free weather underground API key only allows For 10 requests per minute with a maximum of 500 requests per day.

        'Check to see if point needs to be updated
        Dim checkPoint : checkPoint = objPoints.Point(Replace(strFacTag, "::", ":") & "_" & strTempUdc)
        objPoints.UpdateNow(2)

        If (DateDiff("s", checkPoint.Timestamp, Now()) < iUpdateRate And globFunc.IsUpdated(checkPoint.status)) Or checkPoint.facility = "" Then
            'Too soon to update
            ShouldFacilityUpdate = False
        End If

    End Function

    '**********************************************************************************
    ' IsPntCvsQueueEmpty
    ' Notes: Check to see if pnt queues are empty
    '        We need to make sure these queues are empty after adding points to the 
    '        PNT before attempting To issue setpoints
    '**********************************************************************************
    Function IsPntCvsQueueEmpty(strCvsSvc)
        Dim strPntSvc : strPntSvc = globFunc.GetAssociatedPointCfgName(strCvsSvc)

        Dim arrResults As New Object
        Dim iQueuedValue : iQueuedValue = 99

        Do While iQueuedValue > 0
            Call globFunc.GetGenserveInfo(strPntSvc, "PNT_NOTIFY_CVS_Q_CNT", arrResults)
            If arrResults(0, 1) = "" Or arrResults(0, 1) = "0" Then
                iQueuedValue = 0
            Else
                iQueuedValue = CInt(arrResults(0, 1))
            End If
        Loop

        iQueuedValue = 99
        Do While iQueuedValue > 0
            Call globFunc.GetGenserveInfo(strPntSvc, "CVS_CONFIG_QUEUE_CNT", arrResults)
            If arrResults(0, 1) = "" Or arrResults(0, 1) = "0" Then
                iQueuedValue = 0
            Else
                iQueuedValue = CInt(arrResults(0, 1))
            End If
        Loop

        IsPntCvsQueueEmpty = True
    End Function
End Class
