﻿Imports System.Drawing
Imports System.IO
Imports System.Text
Imports CygNet.Data.Historian
Imports CygNet.AE.Shared

Public Class WeatherFileStorageManager
#Region "PROPERTIES"

    Dim VHS As CygNet.Data.Core.DomainSiteService
    Dim name As New Name()
    Dim m_strCvs As String

    Dim m_strWunderApiKey

#End Region

#Region "CONSTRUCTOR"

    Sub New(nDomain As Integer, strVhs As String, strCvs As String, strApiKey As String)
        VHS = New CygNet.Data.Core.DomainSiteService(nDomain, strVhs)
        m_strCvs = strCvs
        m_strWunderApiKey = strApiKey
    End Sub

#End Region

#Region "HELPER METHODS"

    Private Shared Sub DeleteAndReAddName(c As CygNet.API.Historian.Client, n As Name)
        Try
            c.DeleteName(n)
        Catch e As Exception
            Console.WriteLine(e.ToString())
        End Try

        Try
            c.AddName(n, 365)
        Catch e As Exception
            Console.WriteLine(e.ToString())
        End Try
    End Sub


    Private Shared Sub EnsureNameExists(c As CygNet.API.Historian.Client, n As Name)
        Try
            Dim ver = c.GetNameVersion(n)

            Return
        Catch ex As Exception
        End Try

        Try
            c.AddName(n, 365)
        Catch e As Exception
            Console.WriteLine(e.ToString())
        End Try
    End Sub

    Private Shared Function ImgToBytes(file As String) As Byte()
        Return ImgToBytes(Image.FromFile(file))
    End Function

    Private Shared Function ImgToBytes(image As Image) As Byte()
        Dim mem As New MemoryStream()
        image.Save(mem, image.RawFormat)
        Return mem.ToArray()
    End Function

    Private Shared Function BytesToImg(bytes As [Byte]()) As Image
        Dim mem As New MemoryStream(bytes)

        Return Image.FromStream(mem)
    End Function

    ' Given an string array of file paths, the function produces a
    ' HistoricalEntry for each file and adds it to an IEnumerable<HistoricalEntry>
    ' which is subsuquently returned.
    Private Function GenerateEntries(files As String()) As List(Of Tuple(Of HistoricalEntry, Byte()))
        Dim c As New CygNet.API.Historian.Client()
        c.Connect(VHS)

        Dim entries As New List(Of Tuple(Of HistoricalEntry, Byte()))()
        Dim hist As CygNet.Data.Historian.HistoricalEntry

        If files.Length = 0 Then
            ' log and exit
            LogManager.LogMsg("There are no files to add.")

            Return entries
        End If

        Dim dtNow = DateTime.Now

        'create an entry for every file in files array
        For Each img As String In files

            hist = New HistoricalEntry()
            hist.Timestamp = dtNow
            hist.TimeOrdinal = 0
            hist.Value = Encoding.UTF8.GetBytes(img)
            'Stores the full path
            hist.ValueType = CygNet.Data.Historian.HistoricalEntryValueType.UTF8
            Dim bytes = ImgToBytes(Encoding.UTF8.GetString(hist.Value))

            entries.Add(New Tuple(Of HistoricalEntry, Byte())(hist, bytes))

            dtNow = dtNow.AddMilliseconds(1)
        Next

        Return entries
    End Function

    ' Adds all HistoricalEntry supplied by entries to the globally defined VHS.
    Private Sub AddEntriesToVHS(entries As IEnumerable(Of Tuple(Of HistoricalEntry, Byte())))
        Dim ents As New List(Of Tuple(Of HistoricalEntry, Byte()))(entries)

        Dim c As New CygNet.API.Historian.Client()
        c.Connect(VHS)

        'makes sure the value doesn't already exist in the VHS and replace if it does
        DeleteAndReAddName(c, name)
        c.StoreHistoricalEntriesAndBlobs(name, entries)

        LogManager.LogMsg(String.Format("Added {0} entries to VHS", (New List(Of Tuple(Of HistoricalEntry, Byte()))(entries)).Count))
    End Sub


    ' Retrieves EVERY value from the globally specified VHS. 
    ' Returns a Dictionary of Names mapped to Lists of HistoricalEntry
    Private Function RetrieveEntriesFromVHSAndStore(dir As String) As Dictionary(Of Name, List(Of HistoricalEntry))
        Dim allEntries As New Dictionary(Of Name, List(Of HistoricalEntry))()
        Dim retrieved As New List(Of HistoricalEntry)()
        Dim c As New CygNet.API.Historian.Client()
        Dim saveDir As String = Nothing
        Dim img As Image = Nothing

        c.Connect(VHS)
        Directory.CreateDirectory(dir)

        Dim dtStart = New DateTime(2010, 1, 1, 1, 1, 0, DateTimeKind.Local)
        Dim dtEnd = New DateTime(2016, 1, 1, 1, 1, 0, DateTimeKind.Local)

        retrieved = New List(Of HistoricalEntry)()

        'Get all the entries at that Name and add them to a List
        For Each entry In c.GetHistoricalEntriesAndBlobs(name, dtStart, dtEnd, False)
            retrieved.Add(entry.Item1)
            img = BytesToImg(entry.Item2)
            saveDir = Path.Combine(dir, Path.GetFileName(Encoding.UTF8.GetString(entry.Item1.Value)))

            img.Save(saveDir)
        Next

        LogManager.LogMsg(String.Format("Retrieved {0} entries from the VHS", retrieved.Count))
        allEntries.Add(name, retrieved)

        'If there are no entries whatsoever, notify the user and exit
        If allEntries.Count < 1 Then
            LogManager.LogMsg("ERROR: No HistoricalEntry(s) could be retrieved...")
        End If

        Return allEntries
    End Function

    Private Shared Function GetImageDirectory() As String
        Return Path.Combine(Path.GetTempPath(), "WeatherImages")
    End Function

    Private Shared Function GetZipcodeWeatherImageDirectory(zip As String) As String
        Return Path.Combine(GetImageDirectory(), zip)
    End Function

    Private Function GetWeatherGifUri(zip As String) As String
        Return String.Format("http://api.wunderground.com/api/" & m_strWunderApiKey & "/animatedsatellite/q/{0}.gif?" + "newmaps=1&" + "timelabel=1&" + "timelabel.y=10&" + "num=8&" + "delay=50&" + "radius=100", zip)
    End Function

    Private Function GetWeatherGifStream(strZip As String) As Stream
        Return System.Net.HttpWebRequest.CreateHttp(GetWeatherGifUri(strZip)).GetResponse().GetResponseStream()
    End Function

    Private Shared Function StreamToByteArray(s As Stream) As Byte()
        Using ms As New MemoryStream()
            s.CopyTo(ms)
            Return ms.ToArray()
        End Using
    End Function

    Private Sub DownloadWeatherGifToFile(zip As String)
        Dim baseImageDir As String = GetImageDirectory()
        Dim zipcodeImageDir As String = Path.Combine(baseImageDir, zip)

        If Not Directory.Exists(zipcodeImageDir) Then
            Directory.CreateDirectory(zipcodeImageDir)
        End If

        System.Net.HttpWebRequest.CreateHttp(GetWeatherGifUri(zip)).GetResponse().GetResponseStream().CopyTo(File.OpenWrite(zipcodeImageDir & Convert.ToString("\bob.gif")))
    End Sub

    Private Function GenerateNameForZipGif(zip As String) As Name
        Return New Name() With {.ID = String.Format("{1}.{0}WUG:{0}_WUG", zip, m_strCvs)}
    End Function

#End Region

#Region "MAIN METHODS"

    Public Sub RetrieveAndStoreWeatherImage(zip As String)
        Dim c As New CygNet.API.Historian.Client()
        c.Connect(VHS)

        Dim name = GenerateNameForZipGif(zip)

        EnsureNameExists(c, name)

        Dim entry = New HistoricalEntry() With {.Timestamp = DateTime.UtcNow}
        entry.SetValue("gif")

        Dim bytes = StreamToByteArray(GetWeatherGifStream(zip))

        c.StoreHistoricalEntriesAndBlobs(name, New List(Of Tuple(Of HistoricalEntry, Byte()))() From { _
            Tuple.Create(entry, bytes) _
        })
    End Sub

    Public Sub DownloadWeatherImages(zip As String)
        Dim c As New CygNet.API.Historian.Client()
        c.Connect(VHS)

        Dim name = GenerateNameForZipGif(zip)
        Dim dir = GetZipcodeWeatherImageDirectory(zip)

        Directory.CreateDirectory(dir)

        Dim dtStart = New DateTime(2010, 1, 1, 1, 1, 0, DateTimeKind.Local)
        Dim dtEnd = New DateTime(2017, 1, 1, 1, 1, 0, DateTimeKind.Local)

        For Each tuple In c.GetHistoricalEntriesAndBlobs(name, dtStart, dtEnd, False)
            Dim entry = tuple.Item1
            Dim blob = tuple.Item2

            If 0 = entry.BlobLength Then
                Continue For
            End If

            Dim img = BytesToImg(blob)
            Dim filename = Path.Combine(dir, String.Format("{0}.gif", entry.Timestamp.ToString("yyyy-MM-dd-HHmmssfff")))

            img.Save(filename)
        Next
    End Sub

#End Region

End Class
